package App::Memcached::CLI;
use 5.008001;
use strict;
use warnings;

use version; our $VERSION = 'v0.1.0';

our $DEBUG; # set by option

1;
__END__

=encoding utf-8

=head1 NAME

App::Memcached::CLI - It's new $module

=head1 SYNOPSIS

    use App::Memcached::CLI;

=head1 DESCRIPTION

App::Memcached::CLI is ...

=head1 LICENSE

Copyright (C) YASUTAKE Kiyoshi.

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 AUTHOR

YASUTAKE Kiyoshi E<lt>yasutake.kiyoshi@gmail.comE<gt>

=cut

